@@ -13,6 +13,7 @@ import asdf
 from asdf import fits_embed
 from asdf import resolver
 from asdf import schema as asdf_schema
+from asdf.tags.core import NDArrayType
 from asdf.tags.core import ndarray, HistoryEntry
 from asdf import treeutil
 from asdf.util import HashableDict
@@ -155,6 +156,9 @@ def get_hdu(hdulist, hdu_name, index=None):
 
 
 def _make_hdu(hdulist, hdu_name, index=None, hdu_type=None, value=None):
+    if isinstance(value, NDArrayType):
+        value = np.asarray(value)
+
     if hdu_type is None:
         hdu_type = _get_hdu_type(hdu_name, value=value)
         if hdu_type is None:
